/**
 * StandardStudy.java
 *
 * @author Antonio J. Nebro
 * @version 1.0
 */
package jmetal.experiments;

import java.util.logging.Level;
import java.util.logging.Logger;
import java.io.IOException;
import java.util.Properties;
import jmetal.base.Algorithm;
import jmetal.base.Problem;
import jmetal.experiments.settings.*;
import jmetal.experiments.util.RBoxplot;
import jmetal.experiments.util.RWilcoxon;
import jmetal.util.JMException;

/**
 * @author Antonio J. Nebro
 */
public class M1_5Study extends Experiment {

  /**
   * Configures the algorithms in each independent run
   * @param problem The problem to solve
   * @param problemIndex
   * @throws ClassNotFoundException 
   */
  public void algorithmSettings(Problem problem, 
  		                          int problemIndex, 
  		                          Algorithm[] algorithm) throws ClassNotFoundException {
    try {
      int numberOfAlgorithms = algorithmNameList_.length;

      Properties[] parameters = new Properties[numberOfAlgorithms];

      for (int i = 0; i < numberOfAlgorithms; i++) {
        parameters[i] = new Properties();
      } // for

      if (!paretoFrontFile_[problemIndex].equals("")) {
        for (int i = 0; i < numberOfAlgorithms; i++)
          parameters[i].setProperty("paretoFrontFile_", paretoFrontFile_[problemIndex]);
        } // if

        algorithm[0] = new M1_5_NSGAII_Settings(problem).configure(parameters[0]);
        algorithm[1] = new M1_5_ssNSGAII_Settings(problem).configure(parameters[2]);  
        algorithm[2] = new M1_5_SPEA2_Settings(problem).configure(parameters[1]);
        algorithm[3] = new M1_5_SMPSO_Settings(problem).configure(parameters[3]);
        algorithm[4] = new M1_5_IBEA_Settings(problem).configure(parameters[4]);
      } catch (IllegalArgumentException ex) {
      Logger.getLogger(M1_5Study.class.getName()).log(Level.SEVERE, null, ex);
    } catch (IllegalAccessException ex) {
      Logger.getLogger(M1_5Study.class.getName()).log(Level.SEVERE, null, ex);
    } catch  (JMException ex) {
      Logger.getLogger(M1_5Study.class.getName()).log(Level.SEVERE, null, ex);
    }
  } // algorithmSettings

  /**
   * Main method
   * @param args
   * @throws JMException
   * @throws IOException
   */
  public static void main(String[] args) throws JMException, IOException {
    M1_5Study exp = new M1_5Study();

    exp.experimentName_ = "M1.5-Study";
    exp.algorithmNameList_ = new String[]{"NSGAII", "ssNSGAII", "SPEA2", "SMPSO", "IBEA"};
    exp.problemList_ = new String[]{"CTP1", "CTP7", "DEB2DK_k2", "DEB2DK_k4", "DEB3DK_k1", "DEB3DK_k2",
    		"DO2DK_k1_s0", "DO2DK_k4_s1", "DTLZ2", "DTLZ3", "DTLZ4", "DTLZ5", "DTLZ6", "DTLZ7", "DTLZ8",
    		"DTLZ9", "LZ09_F1", "LZ09_F4", "SZDT1", "SZDT2", "CEC2009_UF2", "CEC2009_UF4", "WFG1_2D",
    		"WFG1_3D", "WFG2_2D", "WFG2_3D", "ZDT3", "ZDT4", "ZDT6"};
    exp.paretoFrontFile_ = new String[]{"M1.5.CTP1.pf", "M1.5.CTP7.pf", "M1.5.DEB2DK_k2.pf",
    		"M1.5.DEB2DK_k4.pf", "M1.5.DEB3DK_k1.pf", "M1.5.DEB3DK_k2.pf", "M1.5.DO2DK_k1_s0.pf", 
    		"M1.5.DO2DK_k4_s1.pf", "M1.5.DTLZ2.3D.pf", "M1.5.DTLZ3.3D.pf", "M1.5.DTLZ4.3D.pf",
    		"M1.5.DTLZ5.3D.pf", "M1.5.DTLZ6.3D.pf", "M1.5.DTLZ7.3D.pf", "M1.5.DTLZ8.3D.pf", 
    		"M1.5.DTLZ9.3D.pf", "M1.5.LZ09_F1.pf", "M1.5.LZ09_F4.pf", "M1.5.SZDT1.pf", "M1.5.SZDT2.pf", 
    		"M1.5.UF2.pf", "M1.5.UF4.pf", "M1.5.WFG1.2D.pf", "M1.5.WFG1.3D.pf", "M1.5.WFG2.2D.pf",
    		"M1.5.WFG2.3D.pf", "M1.5.ZDT3.pf", "M1.5.ZDT4.pf", "M1.5.ZDT6.pf"};

    exp.indicatorList_   = new String[] {"HV", "SPREAD", "GD" ,"IGD", "EPSILON"} ;

    int numberOfAlgorithms = exp.algorithmNameList_.length;

    exp.experimentBaseDirectory_ = "data/results/" + exp.experimentName_;
    exp.paretoFrontDirectory_ = "data/M1.5fronts";

    exp.algorithmSettings_ = new Settings[numberOfAlgorithms];

    exp.independentRuns_ = 51;

    // Run the experiments
    int numberOfThreads ;
    exp.runExperiment(numberOfThreads = 4) ;

    // Generate latex tables
    exp.generateLatexTables() ;

    // Configure the R scripts to be generated
    int rows  ;
    int columns  ;
    String prefix ;
    String [] problems ;
    boolean notch ;

    // Configuring scripts for CTP
    rows = 1 ;
    columns = 2 ;
    prefix = new String("CTP");
    problems = new String[]{"CTP1", "CTP7"} ;
    
    exp.generateRBoxplotScripts(rows, columns, problems, prefix, notch = false, exp) ;
    exp.generateRWilcoxonScripts(problems, prefix, exp) ;
    
    // Configuring scripts for knee problems
    rows = 3 ;
    columns = 2 ;
    prefix = new String("");
    problems = new String[]{"DEB2DK_k2", "DEB2DK_k4", "DEB3DK_k1", "DEB3DK_k2",
    		"DO2DK_k1_s0", "DO2DK_k4_s1"} ;
    
    exp.generateRBoxplotScripts(rows, columns, problems, prefix, notch = false, exp) ;
    exp.generateRWilcoxonScripts(problems, prefix, exp) ;
    
    // Configuring scripts for DTLZ
    rows = 4 ;
    columns = 2 ;
    prefix = new String("DTLZ");
    problems = new String[]{"DTLZ2", "DTLZ3", "DTLZ4", "DTLZ5", "DTLZ6", "DTLZ7", "DTLZ8",
    		"DTLZ9"} ;
    
    exp.generateRBoxplotScripts(rows, columns, problems, prefix, notch = false, exp) ;
    exp.generateRWilcoxonScripts(problems, prefix, exp) ;
    
    // Configuring scripts for LZ09
    rows = 1 ;
    columns = 2 ;
    prefix = new String("LZ09");
    problems = new String[]{"LZ09_F1", "LZ09_F4"} ;
    
    exp.generateRBoxplotScripts(rows, columns, problems, prefix, notch = false, exp) ;
    exp.generateRWilcoxonScripts(problems, prefix, exp) ;
    
    // Configuring scripts for SZDT
    rows = 1 ;
    columns = 2 ;
    prefix = new String("SZDT");
    problems = new String[]{"SZDT1", "SZDT2"} ;
    
    exp.generateRBoxplotScripts(rows, columns, problems, prefix, notch = false, exp) ;
    exp.generateRWilcoxonScripts(problems, prefix, exp) ;
    
    // Configuring scripts for UF
    rows = 1 ;
    columns = 2 ;
    prefix = new String("CEC2009");
    problems = new String[]{"CEC2009_UF2", "CEC2009_UF4"} ;
    
    exp.generateRBoxplotScripts(rows, columns, problems, prefix, notch = false, exp) ;
    exp.generateRWilcoxonScripts(problems, prefix, exp) ;
    
    // Configuring scripts for WFG
    rows = 2 ;
    columns = 2 ;
    prefix = new String("WFG");
    problems = new String[]{"WFG1_2D", "WFG1_3D", "WFG2_2D", "WFG2_3D"} ;
    
    exp.generateRBoxplotScripts(rows, columns, problems, prefix, notch = false, exp) ;
    exp.generateRWilcoxonScripts(problems, prefix, exp) ;
    
    // Configuring scripts for ZDT
    rows = 1 ;
    columns = 3 ;
    prefix = new String("ZDT");
    problems = new String[]{"ZDT3", "ZDT4", "ZDT6"} ;
    
    exp.generateRBoxplotScripts(rows, columns, problems, prefix, notch = false, exp) ;
    exp.generateRWilcoxonScripts(problems, prefix, exp) ;

  } // main
} // M1_5Study


