/**
 * CTP1.java
 *
 * @author Marlon A. Braun
 * @version 1.0
 */
package jmetal.problems.CTP;

import jmetal.base.Problem;
import jmetal.base.Solution;
import jmetal.base.SolutionType;
import jmetal.base.solutionType.ArrayRealSolutionType;
import jmetal.base.solutionType.BinaryRealSolutionType;
import jmetal.base.solutionType.RealSolutionType;
import jmetal.util.JMException;
import jmetal.util.wrapper.XReal;

/**
 * This class represents the CTP1 problem for 2 contraints
 */
public class CTP1 extends Problem {

	/**
	 * Creates a default CTP1 problem instance
	 * Default Constructur uses real encoding
	 * @throws ClassNotFoundException
	 */
	public CTP1() throws ClassNotFoundException {
		this("Real");
	}

	/**
	 * Creates a CTP1 problem instance having 2 constraints
	 * @param solutionType The solution type must "Real", "BinaryReal, and "ArrayReal"
	 * @throws ClassNotFoundException
	 */
	public CTP1(String solutionType) throws ClassNotFoundException {
		numberOfVariables_   = 2	  ;
		numberOfObjectives_  = 2	  ;
		numberOfConstraints_ = 2	  ;
		problemName_ 		 = "CTP1" ;
		
		upperLimit_ = new double[numberOfVariables_] ;
	    lowerLimit_ = new double[numberOfVariables_] ;
	    
	    for (int i = 0; i < numberOfVariables_; i++) {
	        lowerLimit_[i] = 0.0 ;
	        upperLimit_[i] = 1.0  ;
	      } // for
	    
	    if (solutionType.compareTo("BinaryReal") == 0)
	    	solutionType_ = new BinaryRealSolutionType(this) ;
	    else if (solutionType.compareTo("Real") == 0)
	    	solutionType_ = new RealSolutionType(this) ;
	    else if (solutionType.compareTo("ArrayReal") == 0)
	    	solutionType_ = new ArrayRealSolutionType(this) ;
	    else {
	    	System.out.println("Error: solution type " + solutionType + " invalid") ;
	    	System.exit(-1) ;
	    }
	}

	/** 
	  * Evaluates a solution 
	  * @param solution The solution to evaluate
	  * @throws JMException 
	  */   
	public void evaluate(Solution solution) throws JMException {
		XReal x = new XReal(solution) ;
        
	    double g;
	    g = 1.0 + x.getValue(1);
	    solution.setObjective(0, x.getValue(0));
	    solution.setObjective(1, g*Math.exp(-solution.getObjective(0)/g));
	} //evaluate
	
	/**
	 * Evaluates the constraints for CTP1
	 */
	public void evaluateConstraints(Solution solution) throws JMException {
		
		double[] obj = new double[2];
		obj[0] = solution.getObjective(0);
		obj[1] = solution.getObjective(1);
		
		double [] constr = new double[numberOfConstraints_];
		constr[0] = obj[1]/(0.858*Math.exp(-0.541*obj[0]))-1.0;
	    constr[1] = obj[1]/(0.728*Math.exp(-0.295*obj[0]))-1.0;
	    
	    double total = 0.0;
	    int number = 0;
	    for (int i = 0; i < numberOfConstraints_; i++) {
	        if (constr[i]<0.0){
	          total+=constr[i];
	          number++;
	        }
	      }
	    
	    solution.setOverallConstraintViolation(total);    
	    solution.setNumberOfViolatedConstraint(number);      
	} //evulateConstraints
}
