/**
 * CTP5.java
 *
 * @author Marlon A. Braun
 * @version 1.0
 */
package jmetal.problems.CTP;

import jmetal.base.Problem;
import jmetal.base.Solution;
import jmetal.base.solutionType.ArrayRealSolutionType;
import jmetal.base.solutionType.BinaryRealSolutionType;
import jmetal.base.solutionType.RealSolutionType;
import jmetal.util.JMException;
import jmetal.util.wrapper.XReal;

/**
 * This class represents the CTP5 problem
 */
public class CTP5 extends Problem {

	/**
	 * Creates a default CTP5 problem instance
	 * Default Constructur uses real encoding
	 * @throws ClassNotFoundException
	 */
	public CTP5() throws ClassNotFoundException{
		this("Real");
	}
	
	/**
	 * Creates a CTP5 problem
	 * @param solutionType The solution type must "Real", "BinaryReal, and "ArrayReal"
	 * @throws ClassNotFoundException
	 */
	public CTP5(String solutionType) throws ClassNotFoundException {
		numberOfVariables_   = 2	  ;
		numberOfObjectives_  = 2	  ;
		numberOfConstraints_ = 1	  ;
		problemName_ 		 = "CTP5" ;
		
		upperLimit_ = new double[numberOfVariables_] ;
	    lowerLimit_ = new double[numberOfVariables_] ;
	    
	    for (int i = 0; i < numberOfVariables_; i++) {
	        lowerLimit_[i] = 0.0 ;
	        upperLimit_[i] = 1.0  ;
	      } // for
	    
	    if (solutionType.compareTo("BinaryReal") == 0)
	    	solutionType_ = new BinaryRealSolutionType(this) ;
	    else if (solutionType.compareTo("Real") == 0)
	    	solutionType_ = new RealSolutionType(this) ;
	    else if (solutionType.compareTo("ArrayReal") == 0)
	    	solutionType_ = new ArrayRealSolutionType(this) ;
	    else {
	    	System.out.println("Error: solution type " + solutionType + " invalid") ;
	    	System.exit(-1) ;
	    }
	}

	/** 
	  * Evaluates a solution 
	  * @param solution The solution to evaluate
	  * @throws JMException 
	  */
	public void evaluate(Solution solution) throws JMException {
		XReal x = new XReal(solution) ;
      
		double[] xreal = new double[2];
		double[] obj = new double[2];
		xreal[0] = x.getValue(0);
		xreal[1] = x.getValue(1);
		
	    double g;
	    g = 1.0 + xreal[1];
	    obj[0] = xreal[0];
	    obj[1] = g*(1.0  - Math.sqrt(obj[0]/g));
	    
	    for(int i = 0 ; i < this.getNumberOfObjectives() ; i++){
	    	solution.setObjective(i, obj[i]);
	    }
	} //evaluate

	/**
	 * Evaluates the constraints
	 */
	public void evaluateConstraints(Solution solution) throws JMException {
		
		double[] obj = new double[2];
		obj[0] = solution.getObjective(0);
		obj[1] = solution.getObjective(1);
		
		double [] constr = new double[numberOfConstraints_];
		double theta, a, b, c, d, e;
	    double exp1, exp2;
	    theta = -0.2*Math.PI;
	    a = 0.1;
	    b = 10.0;
	    c = 2.0;
	    d = 0.5;
	    e = 1.0;
		exp1 = (obj[1]-e)*Math.cos(theta) - obj[0]*Math.sin(theta);
	    exp2 = (obj[1]-e)*Math.sin(theta) + obj[0]*Math.cos(theta);
	    exp2 = b*Math.PI*Math.pow(exp2,c);
	    exp2 = Math.abs(Math.sin(exp2));
	    exp2 = a*Math.pow(exp2,d);
	    constr[0] = exp1/exp2 - 1.0;
	    
	    double total = 0.0;
	    int number = 0;
	    for (int i = 0; i < numberOfConstraints_; i++) {
	        if (constr[i]<0.0){
	          total+=constr[i];
	          number++;
	        }
	      }
	    
	    solution.setOverallConstraintViolation(total);    
	    solution.setNumberOfViolatedConstraint(number);      
	} //evulateConstraints
}
