/**
 * SZDT1.java
 * 
 * @author Marlon A. Braun
 * @version 1.0
 */

package jmetal.problems.SZDT;

import jmetal.base.Problem;
import jmetal.base.Solution;
import jmetal.base.solutionType.ArrayRealSolutionType;
import jmetal.base.solutionType.BinaryRealSolutionType;
import jmetal.base.solutionType.RealSolutionType;
import jmetal.util.JMException;
import jmetal.util.wrapper.XReal;

/**
 * This class represents the SZDT1 problem
 */
public class SZDT1 extends Problem {

	public SZDT1() throws ClassNotFoundException {
		this("Real");
	}
	
	public SZDT1(String solutionType) throws ClassNotFoundException {
		numberOfVariables_   = 30  		;
		numberOfObjectives_  = 2	  	;
		numberOfConstraints_ = 0		;
		problemName_ 		 = "SZDT1"	;
		
		upperLimit_ = new double[numberOfVariables_] ;
	    lowerLimit_ = new double[numberOfVariables_] ;
	    
	    lowerLimit_[0] = 0.7950;    upperLimit_[0] = 1.9500;
	    lowerLimit_[1] = 0.1120;    upperLimit_[1] = 1.2310;
	    lowerLimit_[2] = 0.4220;    upperLimit_[2] = 1.6070;
	    lowerLimit_[3] = 0.4220;    upperLimit_[3] = 1.4860;
	    lowerLimit_[4] = 0.8210;    upperLimit_[4] = 1.8910;
	    lowerLimit_[5] = 0.5590;    upperLimit_[5] = 1.7620;
	    lowerLimit_[6] = 0.2910;    upperLimit_[6] = 1.4570;
	    lowerLimit_[7] = -0.1320;    upperLimit_[7] = 1.0190;
	    lowerLimit_[8] = 0.6020;    upperLimit_[8] = 1.8210;
	    lowerLimit_[9] = 0.3620;    upperLimit_[9] = 1.4450;
	    lowerLimit_[10] = 0.4540;    upperLimit_[10] = 1.6150;
	    lowerLimit_[11] = 0.7350;    upperLimit_[11] = 1.7920;
	    lowerLimit_[12] = 0.7530;    upperLimit_[12] = 1.9220;
	    lowerLimit_[13] = 0.6660;    upperLimit_[13] = 1.7380;
	    lowerLimit_[14] = 0.0410;    upperLimit_[14] = 1.1760;
	    lowerLimit_[15] = 0.2920;    upperLimit_[15] = 1.4060;
	    lowerLimit_[16] = 0.6950;    upperLimit_[16] = 1.9360;
	    lowerLimit_[17] = 0.7880;    upperLimit_[17] = 1.9170;
	    lowerLimit_[18] = 0.1800;    upperLimit_[18] = 1.4100;
	    lowerLimit_[19] = 0.7130;    upperLimit_[19] = 1.8940;
	    lowerLimit_[20] = -0.1370;    upperLimit_[20] = 1.0580;
	    lowerLimit_[21] = 0.2950;    upperLimit_[21] = 1.3530;
	    lowerLimit_[22] = 0.6610;    upperLimit_[22] = 1.8130;
	    lowerLimit_[23] = -0.0970;    upperLimit_[23] = 1.0100;
	    lowerLimit_[24] = 0.0200;    upperLimit_[24] = 1.1390;
	    lowerLimit_[25] = -0.0160;    upperLimit_[25] = 1.2030;
	    lowerLimit_[26] = -0.0040;    upperLimit_[26] = 1.1990;
	    lowerLimit_[27] = 0.3920;    upperLimit_[27] = 1.6040;
	    lowerLimit_[28] = 0.1460;    upperLimit_[28] = 1.2720;
	    lowerLimit_[29] = -0.0390;    upperLimit_[29] = 1.1990;
	    
	    if (solutionType.compareTo("BinaryReal") == 0)
	    	solutionType_ = new BinaryRealSolutionType(this) ;
	    else if (solutionType.compareTo("Real") == 0)
	    	solutionType_ = new RealSolutionType(this) ;
	    else if (solutionType.compareTo("ArrayReal") == 0)
	    	solutionType_ = new ArrayRealSolutionType(this) ;
	    else {
	    	System.out.println("Error: solution type " + solutionType + " invalid") ;
	    	System.exit(-1) ;
	    }
	}
	
	/** 
	  * Evaluates a solution.
	  * @param solution The solution to evaluate
	  * @throws JMException 
	  */
	public void evaluate(Solution solution) throws JMException {
		
		// Initialization
		XReal x = new XReal(solution);
		double[] xreal = new double[numberOfVariables_];
		double[] obj = new double[numberOfObjectives_];
		
		for(int i = 0 ; i < numberOfVariables_ ; i++){
			xreal[i] = x.getValue(i);
		}
		
		// Evaluating
		int nx=30;
		double g;
		double sum=0,psum=0,flag=0, f1, f2;
		double[] z = new double[nx];
		double[] zz = new double[nx];
		double[] p = new double[nx];

		int i;
		double[] o = {0.950, 0.231, 0.607, 0.486, 0.891, 0.762, 0.457, 0.019, 0.821, 0.445, 0.615, 0.792, 0.922, 0.738, 0.176, 0.406, 0.936, 0.917, 0.410, 0.894, 0.058, 0.353, 0.813, 0.010, 0.139, 0.203, 0.199, 0.604, 0.272, 0.199, 0.015, 0.747, 0.445, 0.932, 0.466, 0.419, 0.846, 0.525, 0.203, 0.672, 0.838, 0.020, 0.681, 0.380, 0.832, 0.503, 0.710, 0.429, 0.305, 0.190, 0.193, 0.682, 0.303, 0.542, 0.151, 0.698, 0.378, 0.860, 0.854, 0.594, 0.497, 0.900, 0.822, 0.645, 0.818, 0.660, 0.342, 0.290, 0.341, 0.534, 0.727, 0.309, 0.839, 0.568, 0.370, 0.703, 0.547, 0.445, 0.695, 0.621, 0.795, 0.957, 0.523, 0.880, 0.173, 0.980, 0.271, 0.252, 0.876, 0.737, 0.137, 0.012, 0.894, 0.199, 0.299, 0.661, 0.284, 0.469, 0.065, 0.988};
		double[] d_l = {0.155, 0.119, 0.185, 0.064, 0.070, 0.203, 0.166, 0.151, 0.219, 0.083, 0.161, 0.057, 0.169, 0.072, 0.135, 0.114, 0.241, 0.129, 0.230, 0.181, 0.195, 0.058, 0.152, 0.107, 0.119, 0.219, 0.203, 0.212, 0.126, 0.238, 0.164, 0.236, 0.086, 0.087, 0.188, 0.244, 0.146, 0.124, 0.218, 0.196, 0.120, 0.140, 0.120, 0.207, 0.205, 0.114, 0.107, 0.228, 0.097, 0.208, 0.105, 0.120, 0.135, 0.196, 0.183, 0.111, 0.081, 0.236, 0.183, 0.171, 0.063, 0.160, 0.246, 0.091, 0.064, 0.188, 0.222, 0.082, 0.174, 0.114, 0.155, 0.209, 0.179, 0.130, 0.184, 0.200, 0.108, 0.143, 0.116, 0.155, 0.078, 0.189, 0.128, 0.171, 0.168, 0.226, 0.061, 0.250, 0.201, 0.226, 0.065, 0.194, 0.114, 0.159, 0.135, 0.239, 0.167, 0.204, 0.095, 0.091};
	    double[] lamda_l ={3.236, 4.201, 2.701, 7.775, 7.148, 2.465, 3.020, 3.322, 2.285, 6.004, 3.106, 8.801, 2.956, 6.918, 3.708, 4.403, 2.077, 3.884, 2.171, 2.760, 2.567, 8.636, 3.299, 4.666, 4.208, 2.285, 2.469, 2.362, 3.963, 2.099, 3.052, 2.120, 5.809, 5.753, 2.654, 2.053, 3.427, 4.046, 2.297, 2.548, 4.151, 3.577, 4.165, 2.417, 2.445, 4.405, 4.668, 2.192, 5.147, 2.398, 4.743, 4.157, 3.710, 2.553, 2.733, 4.519, 6.159, 2.121, 2.738, 2.926, 7.918, 3.130, 2.034, 5.495, 7.846, 2.666, 2.253, 6.068, 2.881, 4.388, 3.220, 2.389, 2.791, 3.832, 2.711, 2.496, 4.624, 3.505, 4.297, 3.223, 6.428, 2.644, 3.920, 2.917, 2.978, 2.215, 8.138, 2.000, 2.490, 2.215, 7.711, 2.574, 4.395, 3.135, 3.705, 2.091, 3.003, 2.456, 5.241, 5.499};

		for(i=0;i<nx;i++){
			z[i]=xreal[i]-o[i];
		}

		if ( z[0]>=0){
			zz[0]=z[0];
			f1=zz[0]+1;
		}
		else {
			zz[0]= -lamda_l[0] * z[0];
			f1 = 2.0/(1+Math.exp(z[0]/d_l[0])) * (zz[0]+1);
		}

		for(i=1;i<nx;i++){
			p[i]=0;
			zz[i]=z[i];
			if (z[i]<0)
			{
				zz[i]= -lamda_l[i] * z[i];
				p[i]= -z[i]/d_l[i];
			}
			sum+=zz[i];
			psum = Math.sqrt(Math.pow(psum,2) + Math.pow(p[i],2));
		}
		g = 1+9*sum/(nx-1);
		f2=  (g *(1-Math.sqrt(zz[0]/g)) + 1) * (2.0/(1+Math.exp(-psum)));

	    obj[0] = f1;
	    obj[1] = f2;
	    
	    for(int k = 0 ; k < numberOfObjectives_ ; k++){
	    	solution.setObjective(k, obj[k]);
	    }

	} // evaluate

}
