/**
 * SZDT2.java
 * 
 * @author Marlon A. Braun
 * @version 1.0
 */

package jmetal.problems.SZDT;

import jmetal.base.Problem;
import jmetal.base.Solution;
import jmetal.base.solutionType.ArrayRealSolutionType;
import jmetal.base.solutionType.BinaryRealSolutionType;
import jmetal.base.solutionType.RealSolutionType;
import jmetal.util.JMException;
import jmetal.util.wrapper.XReal;

/**
 * This class represents the SZDT2 problem
 */
public class SZDT2 extends Problem {

	public SZDT2() throws ClassNotFoundException {
		this("Real");
	}
	
	public SZDT2(String solutionType) throws ClassNotFoundException {
		numberOfVariables_   = 30  		;
		numberOfObjectives_  = 2	  	;
		numberOfConstraints_ = 0		;
		problemName_ 		 = "SZDT2"	;
		
		upperLimit_ = new double[numberOfVariables_] ;
	    lowerLimit_ = new double[numberOfVariables_] ;
	    
	    lowerLimit_[0] = 0.4280;    upperLimit_[0] = 1.5830;
	    lowerLimit_[1] = 0.3050;   upperLimit_[1] = 1.4240;
	    lowerLimit_[2] = 0.3310;    upperLimit_[2] = 1.5160;
	    lowerLimit_[3] = 0.2700;    upperLimit_[3] = 1.3340;
	    lowerLimit_[4] = 0.3630;    upperLimit_[4] = 1.4330;
	    lowerLimit_[5] = 0.0230;    upperLimit_[5] = 1.2260;
	    lowerLimit_[6] = 0.4140;    upperLimit_[6] = 1.5800;
	    lowerLimit_[7] = 0.6090;    upperLimit_[7] = 1.7600;
	    lowerLimit_[8] = 0.3110;    upperLimit_[8] = 1.5300;
	    lowerLimit_[9] = 0.5580;    upperLimit_[9] = 1.6410;
	    lowerLimit_[10] = 0.0480;    upperLimit_[10] = 1.2090;
	    lowerLimit_[11] = 0.3230;    upperLimit_[11] = 1.3800;
	    lowerLimit_[12] = 0.6140;    upperLimit_[12] = 1.7830;
	    lowerLimit_[13] = 0.6090;    upperLimit_[13] = 1.6810;
	    lowerLimit_[14] = 0.3260;    upperLimit_[14] = 1.4610;
	    lowerLimit_[15] = 0.4540;    upperLimit_[15] = 1.5680;
	    lowerLimit_[16] = 0.5530;    upperLimit_[16] = 1.7940;
	    lowerLimit_[17] = -0.0700;    upperLimit_[17] = 1.0590;
	    lowerLimit_[18] = 0.3730;    upperLimit_[18] = 1.6030;
	    lowerLimit_[19] = -0.1310;    upperLimit_[19] = 1.0500;
	    lowerLimit_[20] = 0.2200;    upperLimit_[20] = 1.4150;
	    lowerLimit_[21] = 0.2470;    upperLimit_[21] = 1.3050;
	    lowerLimit_[22] = 0.7220;    upperLimit_[22] = 1.8740;
	    lowerLimit_[23] = -0.0920;    upperLimit_[23] = 1.0150;
	    lowerLimit_[24] = 0.6490;    upperLimit_[24] = 1.7680;
	    lowerLimit_[25] = 0.7520;    upperLimit_[25] = 1.9710;
	    lowerLimit_[26] = 0.7870;    upperLimit_[26] = 1.9900;
	    lowerLimit_[27] = 0.5770;    upperLimit_[27] = 1.7890;
	    lowerLimit_[28] = 0.3130;    upperLimit_[28] = 1.4390;
	    lowerLimit_[29] = 0.2600;    upperLimit_[29] = 1.4980;
	    
	    if (solutionType.compareTo("BinaryReal") == 0)
	    	solutionType_ = new BinaryRealSolutionType(this) ;
	    else if (solutionType.compareTo("Real") == 0)
	    	solutionType_ = new RealSolutionType(this) ;
	    else if (solutionType.compareTo("ArrayReal") == 0)
	    	solutionType_ = new ArrayRealSolutionType(this) ;
	    else {
	    	System.out.println("Error: solution type " + solutionType + " invalid") ;
	    	System.exit(-1) ;
	    }
	}
	
	/** 
	  * Evaluates a solution.
	  * @param solution The solution to evaluate
	  * @throws JMException 
	  */
	public void evaluate(Solution solution) throws JMException {
		
		// Initialization
		XReal x = new XReal(solution);
		double[] xreal = new double[numberOfVariables_];
		double[] obj = new double[numberOfObjectives_];
		
		for(int i = 0 ; i < numberOfVariables_ ; i++){
			xreal[i] = x.getValue(i);
		}
		
		// Evaluating
		int nx=30;
		double g;
		double sum=0,psum=0,flag=0, f1, f2;
		double[] z = new double[nx];
		double[] zz = new double[nx];
		double[] p = new double[nx];

		int i;
		double[] o = {0.583, 0.424, 0.516, 0.334, 0.433, 0.226, 0.580, 0.760, 0.530, 0.641, 0.209, 0.380, 0.783, 0.681, 0.461, 0.568, 0.794, 0.059, 0.603, 0.050, 0.415, 0.305, 0.874, 0.015, 0.768, 0.971, 0.990, 0.789, 0.439, 0.498, 0.214, 0.643, 0.320, 0.960, 0.727, 0.412, 0.745, 0.268, 0.440, 0.933, 0.683, 0.213, 0.839, 0.629, 0.134, 0.207, 0.607, 0.630, 0.370, 0.575, 0.451, 0.044, 0.027, 0.313, 0.013, 0.384, 0.683, 0.093, 0.035, 0.612, 0.609, 0.016, 0.016, 0.190, 0.587, 0.058, 0.368, 0.631, 0.718, 0.693, 0.084, 0.454, 0.442, 0.353, 0.154, 0.676, 0.699, 0.728, 0.478, 0.555, 0.121, 0.451, 0.716, 0.893, 0.273, 0.255, 0.866, 0.232, 0.805, 0.908, 0.232, 0.239, 0.050, 0.078, 0.641, 0.191, 0.844, 0.174, 0.171, 0.994};
		double[] d_l = {0.155, 0.119, 0.185, 0.064, 0.070, 0.203, 0.166, 0.151, 0.219, 0.083, 0.161, 0.057, 0.169, 0.072, 0.135, 0.114, 0.241, 0.129, 0.230, 0.181, 0.195, 0.058, 0.152, 0.107, 0.119, 0.219, 0.203, 0.212, 0.126, 0.238, 0.164, 0.236, 0.086, 0.087, 0.188, 0.244, 0.146, 0.124, 0.218, 0.196, 0.120, 0.140, 0.120, 0.207, 0.205, 0.114, 0.107, 0.228, 0.097, 0.208, 0.105, 0.120, 0.135, 0.196, 0.183, 0.111, 0.081, 0.236, 0.183, 0.171, 0.063, 0.160, 0.246, 0.091, 0.064, 0.188, 0.222, 0.082, 0.174, 0.114, 0.155, 0.209, 0.179, 0.130, 0.184, 0.200, 0.108, 0.143, 0.116, 0.155, 0.078, 0.189, 0.128, 0.171, 0.168, 0.226, 0.061, 0.250, 0.201, 0.226, 0.065, 0.194, 0.114, 0.159, 0.135, 0.239, 0.167, 0.204, 0.095, 0.091};
		double[] lamda_l = {3.236, 4.201, 2.701, 7.775, 7.148, 2.465, 3.020, 3.322, 2.285, 6.004, 3.106, 8.801, 2.956, 6.918, 3.708, 4.403, 2.077, 3.884, 2.171, 2.760, 2.567, 8.636, 3.299, 4.666, 4.208, 2.285, 2.469, 2.362, 3.963, 2.099, 3.052, 2.120, 5.809, 5.753, 2.654, 2.053, 3.427, 4.046, 2.297, 2.548, 4.151, 3.577, 4.165, 2.417, 2.445, 4.405, 4.668, 2.192, 5.147, 2.398, 4.743, 4.157, 3.710, 2.553, 2.733, 4.519, 6.159, 2.121, 2.738, 2.926, 7.918, 3.130, 2.034, 5.495, 7.846, 2.666, 2.253, 6.068, 2.881, 4.388, 3.220, 2.389, 2.791, 3.832, 2.711, 2.496, 4.624, 3.505, 4.297, 3.223, 6.428, 2.644, 3.920, 2.917, 2.978, 2.215, 8.138, 2.000, 2.490, 2.215, 7.711, 2.574, 4.395, 3.135, 3.705, 2.091, 3.003, 2.456, 5.241, 5.499};

		for(i=0;i<nx;i++){
			z[i]=xreal[i]-o[i];
		}

		if ( z[0]>=0){
			zz[0]=z[0];
			f1=zz[0]+1;
		}
		else{
			zz[0]= -lamda_l[0] * z[0];
			f1 = 2.0/(1+Math.exp(z[0]/d_l[0])) * (zz[0]+1);
		}

		for(i=1;i<nx;i++){
			p[i]=0;
			zz[i]=z[i];
			if (z[i]<0)
			{
				zz[i]= -lamda_l[i] * z[i];
				p[i]= -z[i]/d_l[i];
			}
			sum+=zz[i];
			psum = Math.sqrt(Math.pow(psum,2) + Math.pow(p[i],2));
		}
		g = 1+9*sum/(nx-1);
		f2 =  (g * (1 - Math.pow(zz[0]/g, 2)) + 1) * (2.0/(1+Math.exp(-psum)));
	
		if ( f1>=f2 ) {
	    obj[0] = f1;
		}
		else {
	    obj[0] = f1;
		}
	    
		obj[1] = f2;
		
	    for(int k = 0 ; k < numberOfObjectives_ ; k++){
	    	solution.setObjective(k, obj[k]);
	    }

	} // evaluate

}
