/**
 * SZDT4.java
 * 
 * @author Marlon A. Braun
 * @version 1.0
 */

package jmetal.problems.SZDT;

import jmetal.base.Problem;
import jmetal.base.Solution;
import jmetal.base.solutionType.ArrayRealSolutionType;
import jmetal.base.solutionType.BinaryRealSolutionType;
import jmetal.base.solutionType.RealSolutionType;
import jmetal.util.JMException;
import jmetal.util.wrapper.XReal;

/**
 * This class represents the SZDT4 problem
 */
public class SZDT4 extends Problem {

	public SZDT4() throws ClassNotFoundException {
		this("Real");
	}
	
	public SZDT4(String solutionType) throws ClassNotFoundException {
		numberOfVariables_   = 30  		;
		numberOfObjectives_  = 2	  	;
		numberOfConstraints_ = 0		;
		problemName_ 		 = "SZDT4"	;
		
		upperLimit_ = new double[numberOfVariables_] ;
	    lowerLimit_ = new double[numberOfVariables_] ;
	    
	    lowerLimit_[0] =  0.8580;    upperLimit_[0] = 1.9570;
	    lowerLimit_[1] = -6.4690;    upperLimit_[1] = 5.4360;
	    lowerLimit_[2] = -5.3940;    upperLimit_[2] = 7.0920;
	    lowerLimit_[3] = -0.8000;   upperLimit_[3] = 10.5230;
	    lowerLimit_[4] = -0.1370;   upperLimit_[4] = 10.6860;
	    lowerLimit_[5] = -2.9030;    upperLimit_[5] = 8.6160;
	    lowerLimit_[6] = -5.0910;    upperLimit_[6] = 6.6460;
	    lowerLimit_[7] =  2.4920;   upperLimit_[7] = 14.4610;
	    lowerLimit_[8] = -6.1910;    upperLimit_[8] = 5.8810;
	    lowerLimit_[9] =  0.6570;   upperLimit_[9] = 12.6060;
	    lowerLimit_[10] = -2.4110;    upperLimit_[10] = 9.4010;
	    lowerLimit_[11] = -2.6440;    upperLimit_[11] = 9.2510;
	    lowerLimit_[12] = -0.3890;   upperLimit_[12] = 10.1820;
	    lowerLimit_[13] = -1.0580;   upperLimit_[13] = 11.3200;
	    lowerLimit_[14] =  3.0570;   upperLimit_[14] = 14.1360;
	    lowerLimit_[15] =  4.1980;   upperLimit_[15] = 14.8710;
	    lowerLimit_[16] =  1.0080;   upperLimit_[16] = 12.3080;
	    lowerLimit_[17] = -0.9080;   upperLimit_[17] = 11.0210;
	    lowerLimit_[18] = -5.1110;    upperLimit_[18] = 6.9410;
	    lowerLimit_[19] = -5.8590;    upperLimit_[19] = 5.6400;
	    lowerLimit_[20] = -6.7010;    upperLimit_[20] = 5.5810;
	    lowerLimit_[21] = -1.7510;    upperLimit_[21] = 9.9700;
	    lowerLimit_[22] = -0.9980;    upperLimit_[22] = 9.6770;
	    lowerLimit_[23] = -1.8390;    upperLimit_[23] = 9.4360;
	    lowerLimit_[24] = -2.2850;    upperLimit_[24] = 8.9970;
	    lowerLimit_[25] = -5.1090;    upperLimit_[25] = 6.9710;
	    lowerLimit_[26] = -6.1070;    upperLimit_[26] = 5.0710;
	    lowerLimit_[27] =  2.3410;   upperLimit_[27] = 13.8800;
	    lowerLimit_[28] =  2.1450;   upperLimit_[28] = 14.4640;
	    lowerLimit_[29] = -1.5200;    upperLimit_[29] = 9.1520;

	    
	    if (solutionType.compareTo("BinaryReal") == 0)
	    	solutionType_ = new BinaryRealSolutionType(this) ;
	    else if (solutionType.compareTo("Real") == 0)
	    	solutionType_ = new RealSolutionType(this) ;
	    else if (solutionType.compareTo("ArrayReal") == 0)
	    	solutionType_ = new ArrayRealSolutionType(this) ;
	    else {
	    	System.out.println("Error: solution type " + solutionType + " invalid") ;
	    	System.exit(-1) ;
	    }
	}
	
	/** 
	  * Evaluates a solution.
	  * @param solution The solution to evaluate
	  * @throws JMException 
	  */
	public void evaluate(Solution solution) throws JMException {
		
		// Initialization
		XReal x = new XReal(solution);
		double[] xreal = new double[numberOfVariables_];
		double[] obj = new double[numberOfObjectives_];
		
		for(int i = 0 ; i < numberOfVariables_ ; i++){
			xreal[i] = x.getValue(i);
		}
		
		// Evaluating
		int nx=numberOfVariables_;
		double g;
		double sum=0,psum=0,flag=0, f1, f2;
		double[] z = new double[nx];
		double[] zz = new double[nx];
		double[] p = new double[nx];

		int i;
		double[] o ={0.957, 0.436, 2.092, 5.523, 5.686, 3.616, 1.646, 9.461, 0.881, 7.606, 4.401, 4.251, 5.182, 6.320, 9.136, 9.871, 7.308, 6.021, 1.941, 0.640, 0.581, 4.970, 4.677, 4.436, 3.997, 1.971, 0.071, 8.880, 9.464, 4.152, 1.318, 4.620, 9.296, 2.804, 9.034, 1.787, 5.197, 7.792, 5.364, 7.301, 0.953, 6.922, 5.955, 5.000, 1.437, 1.800, 2.796, 2.448, 0.499, 2.813, 3.784, 5.816, 7.544, 9.607, 0.634, 7.079, 6.864, 9.367, 2.498, 3.362, 5.484, 8.693, 2.720, 0.246, 1.878, 7.354, 4.399, 8.886, 1.394, 4.045, 7.694, 1.343, 4.430, 4.077, 1.512, 5.488, 7.547, 3.081, 7.321, 7.537, 3.430, 1.710, 9.287, 3.121, 5.341, 1.471, 5.165, 3.627, 7.946, 1.710, 9.013, 7.844, 9.240, 6.567, 4.996, 3.462, 1.847, 2.767, 9.231, 8.492};
		double[] d_l={0.099, 1.905, 2.486, 1.323, 0.823, 1.519, 1.737, 1.969, 2.072, 1.949, 1.812, 1.895, 0.571, 2.378, 1.079, 0.673, 1.300, 1.929, 2.052, 1.499, 2.282, 1.721, 0.675, 1.275, 1.282, 2.080, 1.178, 1.539, 2.319, 0.672, 1.243, 0.883, 0.939, 2.239, 1.249, 1.833, 1.154, 1.773, 1.743, 2.152, 2.445, 1.783, 0.753, 1.610, 1.248, 0.749, 0.703, 1.544, 2.203, 2.355, 1.373, 1.570, 1.330, 0.834, 1.183, 0.731, 1.142, 1.991, 2.101, 1.163, 1.817, 0.849, 1.631, 0.934, 1.672, 1.313, 1.488, 0.826, 1.907, 2.250, 0.676, 0.593, 1.953, 0.699, 1.340, 1.880, 0.690, 1.655, 1.804, 2.296, 1.826, 0.856, 1.924, 1.652, 1.501, 0.903, 1.852, 1.661, 2.351, 2.107, 1.819, 0.574, 0.803, 1.662, 2.390, 2.402, 1.007, 0.654, 1.845, 2.116};
		double[] lamda_l={5.055, 2.625, 2.011, 3.779, 6.077, 3.291, 2.878, 2.540, 2.413, 2.566, 2.760, 2.639, 8.752, 2.103, 4.633, 7.434, 3.847, 2.592, 2.437, 3.336, 2.191, 2.905, 7.409, 3.922, 3.901, 2.404, 4.245, 3.249, 2.156, 7.441, 4.024, 5.665, 5.327, 2.233, 4.003, 2.727, 4.334, 2.820, 2.869, 2.323, 2.045, 2.804, 6.644, 3.105, 4.007, 6.676, 7.116, 3.238, 2.269, 2.123, 3.642, 3.185, 3.759, 5.997, 4.228, 6.837, 4.378, 2.512, 2.380, 4.299, 2.752, 5.893, 3.066, 5.353, 2.990, 3.808, 3.360, 6.055, 2.622, 2.222, 7.394, 8.426, 2.560, 7.155, 3.732, 2.660, 7.246, 3.022, 2.772, 2.178, 2.738, 5.842, 2.599, 3.026, 3.332, 5.538, 2.700, 3.010, 2.126, 2.374, 2.748, 8.707, 6.230, 3.008, 2.092, 2.081, 4.963, 7.649, 2.710, 2.363};

		for(i=0;i<nx;i++)
		{
			z[i]=xreal[i]-o[i];
		}

		if ( z[0]>=0)
		{
			zz[0]=z[0];
			obj[0]=zz[0]+1;
			p[0]=0;
		}
		else
		{
			zz[0]= -lamda_l[0] * z[0];
			p[0] = -z[0]/d_l[0];
			obj[0] = 2.0/(1+Math.exp(-p[0])) * (zz[0]+1);;
		}

		for (i = 1; i < nx; i++)
		{
			p[i]=0;
			zz[i]=z[i];
			if (z[i]<-5)
			{
				zz[i]=-5-lamda_l[i] * (5+z[i]);
				p[i]=(-5-z[i])/d_l[i];
			}
			sum+= zz[i] * zz[i] - 10 * Math.cos(4 * Math.PI * zz[i]);
			psum = Math.sqrt(Math.pow(psum,2) + Math.pow(p[i],2));
		}
		g = 1 + 10 * (nx - 1) + sum;
		obj[1] = (g * (1 - Math.sqrt(zz[0] / g)) + 1) * (2.0/(1+Math.exp(-psum)));
		
	    for(int k = 0 ; k < numberOfObjectives_ ; k++){
	    	solution.setObjective(k, obj[k]);
	    }

	} // evaluate

}
