/**
 * SZDT6.java
 * 
 * @author Marlon A. Braun
 * @version 1.0
 */

package jmetal.problems.SZDT;

import jmetal.base.Problem;
import jmetal.base.Solution;
import jmetal.base.solutionType.ArrayRealSolutionType;
import jmetal.base.solutionType.BinaryRealSolutionType;
import jmetal.base.solutionType.RealSolutionType;
import jmetal.util.JMException;
import jmetal.util.wrapper.XReal;

/**
 * This class represents the SZDT6 problem
 */
public class SZDT6 extends Problem {

	public SZDT6() throws ClassNotFoundException {
		this("Real");
	}
	
	public SZDT6(String solutionType) throws ClassNotFoundException {
		numberOfVariables_   = 30  		;
		numberOfObjectives_  = 2	  	;
		numberOfConstraints_ = 0		;
		problemName_ 		 = "SZDT4"	;
		
		upperLimit_ = new double[numberOfVariables_] ;
	    lowerLimit_ = new double[numberOfVariables_] ;
	    
	    lowerLimit_[0] = 0.2050;    upperLimit_[0] = 1.3600;
	    lowerLimit_[1] = 0.4300;    upperLimit_[1] = 1.5490;
	    lowerLimit_[2] = 0.0770;    upperLimit_[2] = 1.2620;
	    lowerLimit_[3] = 0.5330;   upperLimit_[3] = 1.5970;
	    lowerLimit_[4] = -0.0210;   upperLimit_[4] = 1.0490;
	    lowerLimit_[5] = 0.3680;    upperLimit_[5] = 1.5710;
	    lowerLimit_[6] = 0.5350;    upperLimit_[6] = 1.7010;
	    lowerLimit_[7] = 0.8110;    upperLimit_[7] = 1.9620;
	    lowerLimit_[8] = 0.5320;    upperLimit_[8] = 1.7510;
	    lowerLimit_[9] = 0.6570;    upperLimit_[9] = 1.7400;
	    lowerLimit_[10] = 0.2710;    upperLimit_[10] = 1.4320;
	    lowerLimit_[11] = 0.5770;    upperLimit_[11] = 1.6340;
	    lowerLimit_[12] = 0.6340;    upperLimit_[12] = 1.8030;
	    lowerLimit_[13] = 0.0120;    upperLimit_[13] = 1.0840;
	    lowerLimit_[14] = 0.8100;    upperLimit_[14] = 1.9450;
	    lowerLimit_[15] = 0.8020;    upperLimit_[15] = 1.9160;
	    lowerLimit_[16] = 0.3610;    upperLimit_[16] = 1.6020;
	    lowerLimit_[17] = 0.1250;    upperLimit_[17] = 1.2540;
	    lowerLimit_[18] = 0.6430;    upperLimit_[18] = 1.8730;
	    lowerLimit_[19] = 0.3320;    upperLimit_[19] = 1.5130;
	    lowerLimit_[20] = 0.5380;    upperLimit_[20] = 1.7330;
	    lowerLimit_[21] = 0.3640;    upperLimit_[21] = 1.4220;
	    lowerLimit_[22] = 0.8090;    upperLimit_[22] = 1.9610;
	    lowerLimit_[23] = -0.0350;   upperLimit_[23] = 1.0720;
	    lowerLimit_[24] = 0.4340;    upperLimit_[24] = 1.5530;
	    lowerLimit_[25] = 0.0730;    upperLimit_[25] = 1.2920;
	    lowerLimit_[26] = 0.6550;    upperLimit_[26] = 1.8580;
	    lowerLimit_[27] = 0.1240;    upperLimit_[27] = 1.3360;
	    lowerLimit_[28] = 0.5540;    upperLimit_[28] = 1.6800;
	    lowerLimit_[29] = -0.1850;   upperLimit_[29] = 1.0530;

	    
	    if (solutionType.compareTo("BinaryReal") == 0)
	    	solutionType_ = new BinaryRealSolutionType(this) ;
	    else if (solutionType.compareTo("Real") == 0)
	    	solutionType_ = new RealSolutionType(this) ;
	    else if (solutionType.compareTo("ArrayReal") == 0)
	    	solutionType_ = new ArrayRealSolutionType(this) ;
	    else {
	    	System.out.println("Error: solution type " + solutionType + " invalid") ;
	    	System.exit(-1) ;
	    }
	}
	
	/** 
	  * Evaluates a solution.
	  * @param solution The solution to evaluate
	  * @throws JMException 
	  */
	public void evaluate(Solution solution) throws JMException {
		
		// Initialization
		XReal x = new XReal(solution);
		double[] xreal = new double[numberOfVariables_];
		double[] obj = new double[numberOfObjectives_];
		
		for(int i = 0 ; i < numberOfVariables_ ; i++){
			xreal[i] = x.getValue(i);
		}
		
		// Evaluating
		int nx=numberOfVariables_;
		double g;
		double sum=0,psum=0;
		double[] z = new double[nx];
		double[] zz = new double[nx];
		double[] p = new double[nx];

		int i;
		
		double[] o ={0.360, 0.549, 0.262, 0.597, 0.049, 0.571, 0.701, 0.962, 0.751, 0.740, 0.432, 0.634, 0.803, 0.084, 0.945, 0.916, 0.602, 0.254, 0.873, 0.513, 0.733, 0.422, 0.961, 0.072, 0.553, 0.292, 0.858, 0.336, 0.680, 0.053, 0.357, 0.498, 0.434, 0.562, 0.617, 0.113, 0.898, 0.755, 0.791, 0.815, 0.670, 0.201, 0.273, 0.626, 0.537, 0.060, 0.089, 0.271, 0.409, 0.474, 0.909, 0.596, 0.329, 0.478, 0.597, 0.161, 0.829, 0.956, 0.596, 0.029, 0.812, 0.610, 0.701, 0.092, 0.425, 0.376, 0.166, 0.833, 0.839, 0.452, 0.957, 0.147, 0.870, 0.769, 0.444, 0.621, 0.952, 0.640, 0.247, 0.353, 0.188, 0.491, 0.409, 0.464, 0.611, 0.071, 0.314, 0.608, 0.175, 0.621, 0.246, 0.587, 0.506, 0.465, 0.541, 0.942, 0.342, 0.402, 0.308, 0.412};
		double[] d_l={0.155, 0.119, 0.185, 0.064, 0.070, 0.203, 0.166, 0.151, 0.219, 0.083, 0.161, 0.057, 0.169, 0.072, 0.135, 0.114, 0.241, 0.129, 0.230, 0.181, 0.195, 0.058, 0.152, 0.107, 0.119, 0.219, 0.203, 0.212, 0.126, 0.238, 0.164, 0.236, 0.086, 0.087, 0.188, 0.244, 0.146, 0.124, 0.218, 0.196, 0.120, 0.140, 0.120, 0.207, 0.205, 0.114, 0.107, 0.228, 0.097, 0.208, 0.105, 0.120, 0.135, 0.196, 0.183, 0.111, 0.081, 0.236, 0.183, 0.171, 0.063, 0.160, 0.246, 0.091, 0.064, 0.188, 0.222, 0.082, 0.174, 0.114, 0.155, 0.209, 0.179, 0.130, 0.184, 0.200, 0.108, 0.143, 0.116, 0.155, 0.078, 0.189, 0.128, 0.171, 0.168, 0.226, 0.061, 0.250, 0.201, 0.226, 0.065, 0.194, 0.114, 0.159, 0.135, 0.239, 0.167, 0.204, 0.095, 0.091};
		double[] lamda_l={3.236, 4.201, 2.701, 7.775, 7.148, 2.465, 3.020, 3.322, 2.285, 6.004, 3.106, 8.801, 2.956, 6.918, 3.708, 4.403, 2.077, 3.884, 2.171, 2.760, 2.567, 8.636, 3.299, 4.666, 4.208, 2.285, 2.469, 2.362, 3.963, 2.099, 3.052, 2.120, 5.809, 5.753, 2.654, 2.053, 3.427, 4.046, 2.297, 2.548, 4.151, 3.577, 4.165, 2.417, 2.445, 4.405, 4.668, 2.192, 5.147, 2.398, 4.743, 4.157, 3.710, 2.553, 2.733, 4.519, 6.159, 2.121, 2.738, 2.926, 7.918, 3.130, 2.034, 5.495, 7.846, 2.666, 2.253, 6.068, 2.881, 4.388, 3.220, 2.389, 2.791, 3.832, 2.711, 2.496, 4.624, 3.505, 4.297, 3.223, 6.428, 2.644, 3.920, 2.917, 2.978, 2.215, 8.138, 2.000, 2.490, 2.215, 7.711, 2.574, 4.395, 3.135, 3.705, 2.091, 3.003, 2.456, 5.241, 5.499};

		for(i=0;i<nx;i++){
			z[i]=xreal[i]-o[i];
		}

		if ( z[0]>=0){
			zz[0]=z[0];
			obj[0]=1 - Math.exp(-4.0 *zz[0]) *Math.pow(Math.sin(6.0*Math.PI*zz[0]),6)+1;
		}
		else{
			zz[0]= -lamda_l[0] * z[0];
			obj[0] =  2.0/(1+Math.exp(z[0]/d_l[0])) * (1 - Math.exp(-4.0 *zz[0]) *Math.pow(Math.sin(6.0*Math.PI*zz[0]),6) +1);
		}

		for(i=1;i<nx;i++){
			p[i]=0;
			zz[i]=z[i];
			if (z[i]<0){
				zz[i]=-lamda_l[i] * z[i];
				p[i]=-z[i]/d_l[i];
			}
			sum+=zz[i];
			psum = Math.sqrt(Math.pow(psum,2) + Math.pow(p[i],2));
		}
		g = 1 + 9.0 * Math.pow ( sum/(nx-1),0.25);
		obj[1] =  (g * (1 - Math.pow (( 1 - Math.exp(-4.0 *zz[0]) * Math.pow(Math.sin(6.0*Math.PI*zz[0]),6) )/g,2)) + 1 ) * (2.0/(1+Math.exp(-psum)));

		
	    for(int k = 0 ; k < numberOfObjectives_ ; k++){
	    	solution.setObjective(k, obj[k]);
	    }

	} // evaluate

}
